/**
 * A series of tests to demonstrate async callbacks and potential problems in node.js and mocha
 * 
 * To Run: 
 *     cd test
 *     mocha callbackTest.js
 */
var assert = require('assert');

// Test suite
describe('Testing Callbacks', function(){

    //--------------------------------------------------------

    // test
    it('Synchronous callback', function(){

        // A function
        function greeting(name) {
            console.log('Hello ' + name);
        }

        // A function that takes a function as a parameter
        function processUserInput(callback) {
            var name = "fred";
            callback(name);
        }

        // Run the second function, passing in the first
        processUserInput(greeting);
    });
    //--------------------------------------------------------

    // test - but this does not work... the timeout is cancelled by the end of the test,
    // so the second and third never run.  It works fine in browser tho
    //   https://houssein.me/javascript/2016/05/10/asynchronous-javascript-callbacks.html
    it('Asynchronous callback which fails', function(){

        function functionFirst(callback) {
            setTimeout(function() {
                console.log('Second action');
                callback();
                }, 3000);
        }

        function functionSecond() {
            console.log('Third action');
        }

        functionFirst(function(){
            functionSecond();
        });

        console.log('First action');

    });

    //--------------------------------------------------------

    // Here's how to fix the previous test - use done() with mocha.
    //    https://medium.com/caffeine-and-testing/async-testing-with-mocha-with-callbacks-and-promises-5d0002661b3f
    // However if timeout is greater than 2000, it fails.  
    //    " Error: Timeout of 2000ms exceeded. For async tests and hooks, ensure "done()" is called; if returning a Promise, ensure it resolves."
    it('Asynchronous callback with done', function(done){

        function functionFirst(callback) {
            setTimeout(function() {
                console.log('Second action');
                callback();
                // Mocha tool to wait until done
                done();    // <--- This was the bit I added to wait
                }, 1500);

        }

        function functionSecond() {
            console.log('Third action');
        }

        functionFirst(function(){
            functionSecond();
        });

        console.log('First action');

    });

    //--------------------------------------------------------

    // Here's how to fix the previous test - specify a timeout for mocha.
    // Discussion of this:
    //    https://stackoverflow.com/questions/16607039/in-mocha-testing-while-calling-asynchronous-function-how-to-avoid-the-timeout-er
    //  You can either set the timeout when running your test:
    //        mocha --timeout 15000
    //  Or you can set the timeout for each suite or each test programmatically:
    //        this.timeout(15000);  // add under the describe() or the it()
    it('Asynchronous callback with done and timeout', function(done){

        this.timeout(15000);  // add under the describe() or the it()

        function functionFirst(callback) {
            setTimeout(function() {
                console.log('Second action');
                callback();
                // Mocha tool to wait until done
                done();    // <--- This was the bit I added to wait
                }, 3000);

        }

        function functionSecond() {
            console.log('Third action');
        }

        functionFirst(function(){
            functionSecond();
        });

        console.log('First action');

    });


})